DualSpecQ = LibStub("AceAddon-3.0"):NewAddon("DualSpecQ", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0");
local ldb = LibStub:GetLibrary("LibDataBroker-1.1");
local LibQTip = LibStub("LibQTip-1.0");

DualSpecQ.DQ = {};
local DQ = DualSpecQ.DQ;

local DQicon = "Interface\\Icons\\Inv_Misc_Questionmark"
local tooltip;
local appName = ...

DQ.DEBUG_MODE = true
DQ.VERSION = GetAddOnMetadata(appName, "Version");
DQ.IconFormat = "|T%s:20:20|t";
DQ.SelectSpec = 1;

DQ.Stances = {};
DQ.Stances["WARRIOR"] = {"Stance", {"None", "Battle", "Defensive", "Berserker"}};
--DQ.Stances["DEATHKNIGHT"] = {"Presence", {"None", "Frost", "Blood", "Unholy"}};
--DQ.Stances["PALADIN"] = {"Aura", {"None", "Devotion", "Retribution", "Resistence", "Concentration"}};

DQ.StanceSpells = {};
DQ.StanceSpells["WARRIOR"] = {nil, "Battle Stance", "Defensive Stance", "Berserker Stance"};
--DQ.StanceSpells["DEATHKNIGHT"] = {nil, "Frost Presence", "Blood Presence", "Unholy Presence"};
--DQ.StanceSpells["PALADIN"] = {nil, "Devotion Aura", "Retribution Aura", "Resistence Aura", "Concentration Aura"};

BINDING_HEADER_DUALSPECQ = "DualSpecQ"
_G[ "BINDING_NAME_CLICK DualSpecQBtn:LeftButton" ] = "Change Specialization"

DQ.YesText = "|cff66FF00Yes|r";
DQ.NoText = "|cffff0033No|r";

DQ.PrimaryName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[1]);
DQ.SecondaryName = GetSpellInfo(TALENT_ACTIVATION_SPELLS[2]);
DQ.SwitchTime = 0;

local SpamFilter1 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)');
local SpamFilter2 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)');
local SpamFilter3 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)');

local defaults = {
	char = {
		CustomNames = 1,
		ShowTalent = nil,
		Name1 = nil,
		Name2 = nil,
		Color1 = nil,
		Color2 = nil,
		ShowIcon = nil,
		Stance1 = 1,
		Stance2 = 1,
		Equip1 = 0,
		Equip2 = 0,
	},
	global = {
		HideSpam = true,
	},
}

function DQ.Debug(...)
	if(DQ.DEBUG_MODE) then
		local output, part
		for i=1, select("#", ...) do
			part = select(i, ...)
			part = tostring(part):gsub("{{", "|cffddeeff"):gsub("}}", "|r")
			if (output) then output = output .. " " .. part
			else output = part end
		end
		ChatFrame1:AddMessage(output, 1.0, 1.0, 1.0);
	end
end

function DualSpecQ:RegisterSlashCommands()
	self:RegisterChatCommand("dpq", "ChatCmd");
end

function DualSpecQ:ChatCmd(args)
end

function DualSpecQ:OnInitialize()
	DQ.db = LibStub("AceDB-3.0"):New("DualSpecQDB", defaults, true);
	-- DQ.db.ResetDB();

	DQ.broker = ldb:NewDataObject("DualSpecQ", {
		type = "data source", 
		icon = DQicon, 
		label = "DualSpecQ",
		text = "DualSpecQ",
		OnEnter = function(self, button) DualSpecQ:OnEnter(self, button) end,
	});

	DualSpecQ:RegisterSlashCommands();

	StaticPopupDialogs["DualSpecQName"] = {
	  text = "Enter a name for this specialization",
	  button1 = "Accept",
	  button2 = "Cancel",
	  hasEditBox = 1,
	  OnAccept = function(self)
	      local text = self.editBox:GetText();
	      if text == "" then text = nil end
	        if DQ.SelectSpec == 1 then
			DQ.db.char.Name1 = text;
		else
			DQ.db.char.Name2 = text;
		end
		self:Hide();
		DualSpecQ:Update();
	  end,
	  EditBoxOnEnterPressed = function(self)
	      local text = self:GetParent().editBox:GetText();
	      if text == "" then text = nil end
	        if DQ.SelectSpec == 1 then
			DQ.db.char.Name1 = text;
		else
			DQ.db.char.Name2 = text;
		end
		self:GetParent():Hide();
		DualSpecQ:Update();
	  end,
	  EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide()
	  end,
	  OnHide = function(self)
		self.editBox:SetText("")
	  end,
	  OnShow = function(self)
	        if DQ.SelectSpec == 1 then
			self.editBox:SetText(DQ.db.char.Name1 or "")
		else
			self.editBox:SetText(DQ.db.char.Name2 or "")
		end
		self.editBox:SetFocus()
	  end,
	  timeout = 0,
	  whileDead = true,
	  hideOnEscape = true,
	}
	
end

function DualSpecQ:SecurePreClick(btn, secure)
	if InCombatLockdown() then 
		DualSpecQ:Print("You can't do that while in combat.");
		return 
	end
	if btn ~= "LeftButton" then return end

	local active = GetActiveSpecGroup();

	if active == 1 and DQ.db.char.Stance2 ~= 1 then
		secure:SetAttribute("type", "spell");
		secure:SetAttribute("spell", DQ.StanceSpells[DQ.Class][DQ.db.char.Stance2]);
	elseif active == 2 and DQ.db.char.Stance1 ~= 1 then
		secure:SetAttribute("type", "spell");
		secure:SetAttribute("spell", DQ.StanceSpells[DQ.Class][DQ.db.char.Stance1]);
	end
end

function DualSpecQ:SecurePostClick(btn, secure)
	if InCombatLockdown() then return end
	if btn == "RightButton" then
		DualSpecQ:RightMenu(secure, btn);
	elseif btn == "LeftButton" then
		if tooltip ~= nil then
			tooltip:Hide();
		end
		local active = GetActiveSpecGroup();
		if (active == 1 and DQ.db.char.Stance2 ~= 1) or (active == 2 and DQ.db.char.Stance1 ~= 1) then
			secure:SetAttribute("type", "spell");
			secure:SetAttribute("spell", nil);
		end
		DualSpecQ:ChangeSpec();
	end
end

function DualSpecQ:ChangeEquip(active)
	local equipset, name;

	if active == 1 then
		equipset = DQ.db.char.Equip1;
	else 
		equipset = DQ.db.char.Equip2; 
	end
	
	if equipset ~= 0 then
		name = GetEquipmentSetInfo(equipset);
		if name ~= "" and name ~= nil then
			if Wardrobe ~= nil and Wardrobe.WearOutfit ~= nil then
				Wardrobe.WearOutfit(name);
			else
				UseEquipmentSet(name);
			end
		end
	end
end

function DualSpecQ:OnEnable()
	self:RegisterEvent("ACTIVE_TALENT_GROUP_CHANGED");
	self:RegisterEvent("PLAYER_LEAVING_WORLD");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_TALENT_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");

	DualSpecQ:ScheduleTimer("Update", 0.01);

	CreateFrame("Button", "DualSpecQBtn", UIParent, "SecureActionButtonTemplate");

	DualSpecQBtn:SetScript('PreClick', function (_, btn) 
		DualSpecQ:SecurePreClick(btn, DualSpecQBtn);
	end);
	DualSpecQBtn:SetScript('PostClick', function (_, btn) 
		DualSpecQ:SecurePostClick(btn, DualSpecQBtn);
	end);
	DualSpecQBtn:SetScript("OnEnter", function(self) 
		DualSpecQ:OnEnterSecure(self, button);
	end);
	DualSpecQBtn:RegisterForClicks('AnyUp');
	DualSpecQBtn:Hide();
end

function DualSpecQ:PLAYER_LEAVING_WORLD(event)
	self:UnregisterEvent("PLAYER_TALENT_UPDATE")
end

function DualSpecQ:PLAYER_ENTERING_WORLD(event)
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
	DQ.Class = select(2, UnitClass("player"));
end

function DualSpecQ:ACTIVE_TALENT_GROUP_CHANGED(event, group)
	DualSpecQ:Print("Switched to "..DQ.CurrentTalentName..".");
	DualSpecQ:ScheduleTimer("ChangeEquip", 0.1, GetActiveSpecGroup());
end

function DualSpecQ:PLAYER_TALENT_UPDATE(event)
	DualSpecQ:Update();
end

function DualSpecQ:UNIT_SPELLCAST_START(event, ...)
	local unit, spellName = ...
	if DQ.db.global.HideSpam == true and unit == "player" and (spellName == DQ.PrimaryName or spellName == DQ.SecondaryName) then
		DQ.SwitchTime = GetTime();
		ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", DQ.SpamFilter);
	end
end

function DualSpecQ:UNIT_SPELLCAST_INTERRUPTED(event, ...)
	local unit, spellName = ...
	if unit == "player" and (spellName == DQ.PrimaryName or spellName == DQ.SecondaryName) then
		if DQ.db.global.HideSpam == true then
			ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", DQ.SpamFilter);
		end
	end
end

function DualSpecQ:Update()
	local name, icon, talenttext, _;

	local TalentGroup = GetActiveSpecGroup();

	local TabId = GetSpecialization(false, false, TalentGroup);
	local colors = DQ.GetColorSpec(TalentGroup);

	if TabId ~= nil then
		_, name, _, icon = GetSpecializationInfo(TabId, false, false, TalentGroup);

		local talents = {};
		for i = 1, 3 do
			_,_,_,_,talents[i] = GetSpecializationInfo(i, false, false, TalentGroup);
		end
		talenttext = "("..talents[1].."/"..talents[2].."/"..talents[3]..")";
	else
		name = "Unknown"
		talenttext = "(No Talents!)"
		icon = DQicon;

	end
	DQ.broker.icon = icon;
	if DQ.db.char.CustomNames ~= nil then
		if TalentGroup == 1 and DQ.db.char.Name1 ~= nil then
			name = DQ.db.char.Name1;
		elseif TalentGroup == 2 and DQ.db.char.Name2 ~= nil then
			name = DQ.db.char.Name2;
		end
	end
	DQ.broker.text = DualSpecQ:ColorText(name, colors[1], colors[2], colors[3]);
	DQ.CurrentTalentName = DQ.broker.text;
	if DQ.db.char.ShowTalent ~= nil then
		DQ.broker.text = DQ.broker.text.." "..talenttext;
	end
end

function DQ.ColorSelect(restore)
	local newR, newG, newB, newA;
	if restore then
  		newR, newG, newB = unpack(restore);
 	else
  		-- Something changed
  		newR, newG, newB = ColorPickerFrame:GetColorRGB();
 	end
 
 	-- Update our internal storage.
	local Colors;

	if DQ.SelectSpec == 1 then 
		DQ.db.char.Color1 = {newR, newG, newB};
	else
		DQ.db.char.Color2 = {newR, newG, newB};
	end
	DualSpecQ:Update();
end	

function DQ.GetColorSpec(spec)
	local r, b, g = 1, 1, 1;
	if spec == 1 then
		Colors = DQ.db.char.Color1;
	else
		Colors = DQ.db.char.Color2;
	end
	if Colors ~= nil then
		r = Colors[1];
		g = Colors[2];
		b = Colors[3];
	end
	return {r, g, b};
end

function DQ.ColorPick(self, spec)
	local colors = DQ.GetColorSpec(spec);
	DQ.SelectSpec = spec;
	r = colors[1];
	g = colors[2];
	b = colors[3];

	DQ.ColorSpec = spec;
	ColorPickerFrame:SetColorRGB(r,g,b);
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = false, nil;
	ColorPickerFrame.previousValues = {r,g,b};
	ColorPickerFrame.func, ColorPickerFrame.cancelFunc = DQ.ColorSelect, DQ.ColorSelect;
	ColorPickerFrame:Hide(); -- Need to run the OnShow handler.
	ColorPickerFrame:SetFrameStrata("HIGH");
	ColorPickerFrame:Show();
end
		
function DualSpecQ:ChangeSpec()
	local active = GetActiveSpecGroup();
	if GetActiveSpecGroup() < 2 then
		DualSpecQ:Print("You do not have an alternate specialization.");
	end
	SetActiveSpecGroup(-active + 3);
	return;
end

function DQ.SelectTalent(self, info)
	local text;
	if DQ.db.char.ShowTalent ~= nil then
		DQ.db.char.ShowTalent = nil;
		text = DQ.NoText;
	else
		DQ.db.char.ShowTalent = 1;
		text = DQ.YesText;
	end
	info[1]:SetCell(info[2], 3, text);
	DualSpecQ:Update();
end

function DQ.SelectEquip(self, info)
	local text;
	local maxequip = GetNumEquipmentSets();
	local setnum;

	if info[3] == 1 then
		setnum = DQ.db.char.Equip1;
	elseif info[3] == 2 then
		setnum = DQ.db.char.Equip2;
	end
	setnum = setnum + 1;
	if setnum > maxequip then 
		setnum = 0;
		text = "None"
	else
		text = GetEquipmentSetInfo(setnum);
	end
	info[1]:SetCell(info[2], 3, text);
	if info[3] == 1 then
		DQ.db.char.Equip1 = setnum;
	elseif info[3] == 2 then
		DQ.db.char.Equip2 = setnum;
	end
	DualSpecQ:Update();
end

function DQ.SelectStance(self, info)
	local text;
	local stances = DQ.Stances[DQ.Class][2];
	if info[3] == 1 then
		DQ.db.char.Stance1 = DQ.db.char.Stance1 + 1;
		if stances[DQ.db.char.Stance1] == nil then
			DQ.db.char.Stance1 = 1;
		end
		text = stances[DQ.db.char.Stance1];
	elseif info[3] == 2 then
		DQ.db.char.Stance2 = DQ.db.char.Stance2 + 1;
		if stances[DQ.db.char.Stance2] == nil then
			DQ.db.char.Stance2 = 1;
		end
		text = stances[DQ.db.char.Stance2];
	end
	info[1]:SetCell(info[2], 3, text);
	DualSpecQ:Update();
end

function DQ.SelectCustom(self, info)
	local text;
	if DQ.db.char.CustomNames ~= nil then
		DQ.db.char.CustomNames = nil;
		text = DQ.NoText;
	else
		DQ.db.char.CustomNames = 1;
		text = DQ.YesText;
	end
	info[1]:SetCell(info[2], 3, text);
	DualSpecQ:Update();
end

function DQ.SelectSpam(self, info)
	local text;
	if DQ.db.global.HideSpam ~= nil then
		DQ.db.global.HideSpam = nil;
		text = DQ.NoText;
	else
		DQ.db.global.HideSpam = true;
		text = DQ.YesText;
	end
	info[1]:SetCell(info[2], 3, text);
	DualSpecQ:Update();
end

function DualSpecQ:RightMenu(self, button)
	local format = "%s";
	local text;
	format = "|cffffc1c1"..format.."|r";
	local PrimaryText = "Primary"
	local SecondaryText = "Secondary"
	local TalentGroup = GetActiveSpecGroup();

	if TalentGroup == 1 then 
		PrimaryText = "Current"
	else
		SecondaryText = "Current"
	end

	GameTooltip:Hide();
	tooltip = LibQTip:Acquire("DualSpecQTooltip", 3, "LEFT", "LEFT");
	tooltip:SetScale(1.1);
	tooltip:SmartAnchorTo(self)
	
	tooltip:Clear();

	local line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "|cff00E5EEDualSpecQ|r", nil, "CENTER", 3);
	tooltip:AddSeparator();
	
	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Allow Custom Spec Names", nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectCustom, {tooltip, line});
	if DQ.db.char.CustomNames ~= nil then 
		text = DQ.YesText;
	else
		text = DQ.NoText;
	end
	tooltip:SetCell(line, 3, text);

	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Hide Talent Spam", nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectSpam, {tooltip, line});
	if DQ.db.global.HideSpam ~= nil then 
		text = DQ.YesText;
	else
		text = DQ.NoText;
	end
	tooltip:SetCell(line, 3, text);

	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Show Talent Points", nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectTalent, {tooltip, line});
	if DQ.db.char.ShowTalent ~= nil then 
		text = DQ.YesText;
	else
		text = DQ.NoText;
	end
	tooltip:SetCell(line, 3, text);

	tooltip:AddSeparator();

	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Rename "..PrimaryText.." Spec", nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SpecRename, 1);

	local colors = DQ.GetColorSpec(1);
	text = DualSpecQ:ColorText("Change "..PrimaryText.." Spec Color", colors[1], colors[2], colors[3]);
	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, text, nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.ColorPick, 1);

	local equipsets = GetNumEquipmentSets();
	if equipsets > 0 then
		local equip = DQ.db.char.Equip1;

		-- Error check since newly added
		if equip == nil then equip = 0; end

		local equipname = "None"
		if equip > 0 then
			equipname = GetEquipmentSetInfo(equip);
			if equipname == nil or equipname == "" then
				DQ.db.char.Equip1 = 1;
				equipname = "None";
			end
		end
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Equipment Set:", nil, "LEFT", 1);
		tooltip:SetCell(line, 3, equipname, nil, "LEFT", 1);
		tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectEquip, {tooltip, line, 1});
	end

	local stance = DQ.Stances[DQ.Class];

	if DQ.Stances[DQ.Class] ~= nil then
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Change "..PrimaryText.." "..stance[1]..":", nil, "LEFT", 1);
		tooltip:SetCell(line, 3, stance[2][DQ.db.char.Stance1], nil, "LEFT", 1);
		tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectStance, {tooltip, line, 1});
	end
		
	tooltip:AddSeparator();

	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Rename "..SecondaryText.." Spec", nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SpecRename, 2);

	colors = DQ.GetColorSpec(2);
	text = DualSpecQ:ColorText("Change "..SecondaryText.." Spec Color", colors[1], colors[2], colors[3]);
	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, text, nil, "LEFT", 1);
	tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.ColorPick, 2);

	local equipsets = GetNumEquipmentSets();
	if equipsets > 0 then
		local equip = DQ.db.char.Equip2;

		-- Error check since newly added
		if equip == nil then equip = 0; end

		local equipname = "None"
		if equip > 0 then
			equipname = GetEquipmentSetInfo(equip);
			if equipname == nil or equipname == "" then
				DQ.db.char.Equip2 = 1;
				equipname = "None";
			end
		end
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Equipment Set:", nil, "LEFT", 1);
		tooltip:SetCell(line, 3, equipname, nil, "LEFT", 1);
		tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectEquip, {tooltip, line, 2});
	end

	if DQ.Stances[DQ.Class] ~= nil then
		line = tooltip:AddLine();
		tooltip:SetCell(line, 1, "Change "..SecondaryText.." "..stance[1]..":", nil, "LEFT", 1);
		tooltip:SetCell(line, 3, stance[2][DQ.db.char.Stance2], nil, "LEFT", 1);
		tooltip:SetCellScript(line, 1, "OnMouseDown", DQ.SelectStance, {tooltip, line, 2});
	end
		
	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.25, self)
	tooltip:UpdateScrolling()
	tooltip:Show()
end

function DQ.SpecRename(self, spec)
	DQ.SelectSpec = spec;
	StaticPopup_Show("DualSpecQName");
end

function DualSpecQ:OnEnter(brokerframe, button)
	if InCombatLockdown() then return end

	DualSpecQBtn:RegisterForDrag("LeftButton")
	DualSpecQBtn:SetMovable(true)
	DualSpecQBtn:SetScript("OnDragStart", function(self)
		if InCombatLockdown() then return end
		if GameTooltip ~= nil then GameTooltip:Hide() end
		brokerframe:GetScript("OnDragStart")(brokerframe)
		self:SetScript("OnUpdate", function(self)
			self:SetAllPoints(brokerframe)
			if not IsMouseButtonDown("LeftButton") then
				brokerframe:GetScript("OnDragStop")(brokerframe)
				self:SetScript("OnUpdate",nil)
			end
		end)
		self:Show()
	end)

	brokerframe:SetScript("OnHide", function()
		if not InCombatLockdown() then DualSpecQBtn:Hide(); end
	end)

	DualSpecQBtn:SetFrameStrata(brokerframe:GetFrameStrata())
	DualSpecQBtn:SetFrameLevel(brokerframe:GetFrameLevel()+1)
	DualSpecQBtn:SetAllPoints(brokerframe)
	DualSpecQBtn:Show()
end



-- Rename
function DualSpecQ:OnEnterSecure(self, button)
	GameTooltip:Hide();
	tooltip = LibQTip:Acquire("DualSpecQTooltip", 3, "LEFT", "LEFT");
	tooltip:SetScale(1.0);
	tooltip:SmartAnchorTo(self)
	
	tooltip:Clear();

	local line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "|cff00E5EEDualSpecQ|r", nil, "CENTER", 3);
	tooltip:AddSeparator();

	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Left Click to Change Specs", nil, "CENTER", 3);
	line = tooltip:AddLine();
	tooltip:SetCell(line, 1, "Right Click for Options", nil, "CENTER", 3);

	tooltip:EnableMouse()
	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.05, self)
	tooltip:UpdateScrolling()
	tooltip:Show()
end

function DualSpecQ:ColorText(s, r, g, b)
	return "|c"..string.format("%02x%02x%02x%02x", 255, r*255, g*255, b*255)..s.."|r";
end

function DQ.SpamFilter(self, event, msg)
	local CheckTime = GetTime() - 8
	if (DQ.SwitchTime > CheckTime) and ((msg:find(SpamFilter1) ~= nil or msg:find(SpamFilter2) ~= nil or msg:find(SpamFilter3) ~= nil)) then
		return true
	end
	if DQ.SwitchTime < CheckTime then
		ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", DQ.SpamFilter);
	end
end




